; 				Chip Addiction
; 			        /Mic, 2005/2008
;
;   		    http://jiggawatt.org/badc0de  |  micol972@gmail.com


; Assemble with NESASM 2.51


; RAM page 3 is used for variables
; RAM page 4 is used for sprite data



 .inesprg    2			; Two 16k PRG-ROM banks
 .ineschr    1			; One 8k CHR-ROM bank
 .inesmir    1			; Vertical mirroring
 .inesmap    0			; Mapper 0 (none)


__VRC6	=	%00000001
__VRC7	=	%00000010
__FDS	=	%00000100
__MMC5	=	%00001000
__N106	=	%00010000
__FME7	=	%00100000


; Zeropage variables
STRPTRL 	= $C0
STRPTRH 	= $C1
fubar 		= $C2

CYCCTR		= $D0
DETMOD		= $D2


; Various definitions
CUBEY 		= 108
TILOFS 		= 0



; Variables. Just picked a random RAM address not used by the MCK driver.
 .org $06C0
 	flipflop: 	.db 0	; Used to slow down scrolling a bit
 	counter:	.db 0	; Used as a counter when both X & Y are needed for other things
 	permit_read:	.db 0
 	
 	fade_ctr:	.db 0
 	fade_val:	.db 0
 	fade_dir:	.db 0
 	
 	twirl_pos:	.db 0	; Used for controlling the twirling image
 	twirl_pos2:	.db 0
 	
 	scL:		.db 0	; Used for the scroller
 	scH:		.db 0
 	scL2:		.db 0
 	scH2:		.db 0
 	
 	scrollX:	.db 255
 	scrollXH:	.db 0
  	
  	curr_str:	.db 0
 	counter2:	.db 0
 	counter3:	.db 0
	counter4:	.db 0
	counter5:	.db 0
	counter6:	.db 0
	counter7:	.db 0
 	counter8:	.db 0
 	counter9:	.db 0
 	counter10:	.db 0
 	counter11:	.db 0
 	speed:		.db 0
 	
 	palntsc:	.db 0
 	
 	volumes:	.db 0,0,0,0
 	

 .bank 0
 .org  $8000
 
	; Sound data
	.include	"ppmck/define.inc"
	.include	"effect.h"
	.include	"ppmck/sounddrv.h"
	.include	"ppmck/dpcm.h"
	.include	"ppmck/internal.h"
	.include	"ppmck/freqdata.h"
	.include	"macros.inc"




; Setup the sprite data for the n:th vector ball
draw_ball .macro
	ldy 	idx\1,x
	lda 	ylut,y		; Get vertical position
	clc
	adc 	#CUBEY		; Add vertical base
	sta 	$480 + \1*4
	lda 	#1
	sta 	$481 + \1*4
	lda 	#1
	sta 	$482 + \1*4
	lda 	xlut,y		; Get horizontal position
	clc
	adc 	counter7
	sta 	$483 + \1*4
 .endm
 
 
 
		
 .bank 2
 .org $C000			


;------------------
;      RESET
;------------------

reset:
	cld
	sei
	ldx 	#$00
	stx 	REG_PPUCNT1	; No NMI
	stx 	REG_PPUCNT2	; Disable screen
	inx
waitvb:	
	lda 	REG_PPUSTAT
	bpl 	waitvb		; Wait a few frames
	dex
	bpl 	waitvb
	txs			; Set up stack pointer

	lda	#0
	sta	palntsc
	
; Try to detect PAL or NTSC
redo:
	lda 	#0
	sta 	<CYCCTR		; Clear counter	
	sta 	<CYCCTR+1	; ...
detect_mode:
	inc 	<CYCCTR
	bne 	no_cyc
	inc 	<CYCCTR+1
no_cyc:
	lda 	REG_PPUSTAT
	bpl 	detect_mode	; Wait until there's a vblank
	lda 	<CYCCTR+1
	cmp 	#9
	bpl 	redo		; The loop probably lasted longer than one frame, try again
	cmp 	#8
	bmi 	ntsc		; Too few cycles to be PAL
	lda 	<CYCCTR
	cmp 	#$32
	bmi 	ntsc
	jmp 	pal

ntsc:
	lda 	#1
	sta 	palntsc
	jmp 	pal

pal:	
	; Initialise sound player
	jsr 	sound_init

	; Setup nametables and attribute tables	
	jsr 	setup
	jsr 	copy_gfx

	; Turn the screen back on
	jsr 	enable_screen

	; Init a bunch of variables
	movi 	scrollX,255
	clr 	scrollXH
	clr 	curr_str
	
	; These two control the scrolltext color cycling
	movi 	counter3,40
	clr 	counter4
	
	; Pointer to current string being scrolled
	movi16 	STRPTRL,szMessage
	
	; These control the cube's angle and horizontal position 
	clr 	counter5
	clr 	counter6
	clr 	counter8
	movi 	speed,1
	
	; Number of frames until we change the cube rotation
	movi 	counter9,128
	
	lda 	#0
	sta 	current_envelope
	sta 	current_envelope+2
	sta 	current_envelope+4
	sta 	current_envelope+6
	

	; Main loop	
forever:
	lda 	#0
	sta 	current_envelope+4
	jsr 	sound_driver_start	; Update music

	; Update sine scroller	
	jsr 	scroller
	
 	; Wait for the sprite0 hit flag to go low
wait_sprite0_clear:
	bit 	REG_PPUSTAT
	bvs 	wait_sprite0_clear

	; Increment starting position in distortion table every
	; other frame.
	ldx 	twirl_pos
	lda 	flipflop
	eor 	#1
	sta 	flipflop
	bne 	noinc
	inx
noinc:
	txa
	and 	#$3F
	sta 	twirl_pos
	sta 	twirl_pos2

	ldy 	#64

	; Wait for the sprite0 hit, which will be our key to start
	; doing the sine-distortion of the logo.
wait_sprite0_hit:
	bit 	REG_PPUSTAT
	bvc 	wait_sprite0_hit

	lda palntsc
	cmp #1
	beq scroll_lines_ntsc
	
	; This code distorts the logo
	; Timed to be 106+5/9 cycles per iteration
scroll_lines:
	ldx 	twirl_pos2	;4
	inx			;2
	txa			;2
	and 	#$3F		;2
	sta 	twirl_pos2	;4
	tax			;2
	lda 	sin_64_12,x	;4
	clc			;2
	adc 	#9		;2
	tax			;2 (26)

	stx 	$2005		;4
	lda 	#0		;2
	sta 	$2005		;4 (10)

	ldx 	#10		;2
delay2:	dex 			;2
	bne 	delay2		;3 / 2

	nop			;2
	lda 	fubar		;4
	adc 	#113		;2
	bcc 	delay3		;3 / 2
delay3:			
	sta 	fubar		;4
	
	dey			;2
	bne 	scroll_lines	;3

	lda 	#0
	sta 	$2005
	sta 	$2005
		
	jmp 	forever	; repeat..


	; This code distorts the logo
	; Timed to be 113+1/3 cycles per iteration
scroll_lines_ntsc:
	ldx 	twirl_pos2	;4
	inx			;2
	txa			;2
	and 	#$3F		;2
	sta 	twirl_pos2	;4
	tax			;2
	lda 	sin_64_12,x	;4
	clc			;2
	adc 	#9		;2
	tax			;2 (26)

	stx 	$2005		;4
	lda 	#0		;2
	sta 	$2005		;4 (10)

	ldx 	#11		;2
delay4:	dex 			;2
	bne 	delay4		;3 / 2

	nop			;2
	lda 	fubar		;4
	adc 	#88		;2
	bcc 	delay5		;3 / 2
delay5:			
	nop			;2
	nop			;2
	nop			;2
	
	dey			;2
	bne 	scroll_lines_ntsc ;3

	lda 	#0
	sta 	$2005
	sta 	$2005
		
	jmp 	forever	; repeat..



;--------------------------------
;    Copy palette data to VRAM
;--------------------------------
refresh_palette:
	set_vram_adr $3F00
	memcpy_fixed_dest REG_VRAMIO,$580,32
	rts
	

;-----------------------------------------
;  Clear nametables and attribute tables
;-----------------------------------------
setup:
	lda 	#$0
	
	; Reset scrolling registers
	sta 	$2005
	sta 	$2005
	
	memset_page $300,#0
	memset_page $400,#0
	
	CLS 	0,#($8E + TILOFS)
	CLS 	1,#($8E + TILOFS)
	
	; Clear attribute tables
	set_vram_adr $23C0
	memset_fixed_dest $2007,#$55,$40
	set_vram_adr $27C0
	memset_fixed_dest $2007,#$00,$40

	; Copy palette data from ROM to RAM	
	memcpy 	$580,palette,$20
	
	jsr 	refresh_palette
	
	movi 	fade_ctr,16
	movi 	fade_val,1
	clr 	fade_dir
	
	rts



;-----------------------------------------
;  Setup nametables and attribute tables
;-----------------------------------------
copy_gfx:
	; Setup nametable 0
	set_vram_adr $2000
	
	; ..start copying from ROM..
	memcpy_page_fixed_dest REG_VRAMIO,nametable_1
	
	memset_fixed_dest REG_VRAMIO,#($8E+TILOFS),64
	memset_fixed_dest REG_VRAMIO,#(150+TILOFS),32
	memset_fixed_dest REG_VRAMIO,#(149+TILOFS),32
	
	lda 	#(143 + TILOFS)
	ldy 	#5
set_nt_outer:
	ldx 	#32
set_nt_1:
	sta 	REG_VRAMIO
	dex
	bne 	set_nt_1
	tax
	inx
	txa
	dey
	bne 	set_nt_outer

	lda 	#(148 + TILOFS)
	ldy 	#13
set_nt_outer_2:
	ldx 	#32
set_nt_2:
	sta 	REG_VRAMIO
	dex
	bne 	set_nt_2
	dey
	bne 	set_nt_outer_2
	
	; Setup attributetable 0
	set_vram_adr $23C0
	memcpy_fixed_dest REG_VRAMIO,attrtable_1,$10
	memset_fixed_dest REG_VRAMIO,#$50,8
		
	; Setup sprites
	memcpy 	$400,sprite_data,4

	; Reset VRAM pointer
	set_vram_adr $2000

	rts
	

	
	
enable_screen:
	w_vbi_e:
		lda 	REG_PPUSTAT
	        bpl 	w_vbi_e

	; Enable BG & sprites, don't clip BG
	lda 	#(PPUCNT2_E_OBJ | PPUCNT2_E_BG | PPUCNT2_NOCLIP_BG | PPUCNT2_CLIP_OBJ | PPUCNT2_COLOR)
	sta 	REG_PPUCNT2  

	; Enable NMI, select upper VROM bank for both BG & sprite patterns
	lda 	#(PPUCNT1_E_NMI | PPUCNT1_BG_0000 | PPUCNT1_OBJ_1000 | PPUCNT1_INC_1 | PPUCNT1_NT_2000)
	sta 	REG_PPUCNT1

	rts
	

;------------------------
;  Handle sine scroller
;------------------------
scroller:
	ldx 	#0
	stx 	counter
	mov16 	scL,scrollX
	
scr_loop:
	ldy 	counter
	lda 	[STRPTRL],y	; Get character 
	beq 	scr_done	; Null terminator ?
	sta 	$405,x		; Set character index. The character set ranges from 32-127, so the
				; tile data is stored beginning at tile #32 in chr bank 1, meaning
				; the character numbers directly correspond to tile numbers.
		
	lda 	scH
	beq 	x_ok		; Check high byte of horizontal position
	lda 	#0
	sta 	$407,x
	lda 	#8
	sta 	$404,x
	jmp 	not_visible
x_ok:
		
	lda 	scL		; Low byte of horizontal position
	tay
	sta 	$407,x		; X coordinate of sprite
	
	lda 	sin256,y	; Get the Y position from the sine table
	sta 	$404,x		; Store in the sprite buffer

	lda 	#0		; Sprite attributes... none
	sta 	$406,x

not_visible:
	
	; Update horizontal position. I use 9 pixels between each character.
	lda 	scL
	add 	#9
	sta 	scL
	bcc 	no_carry_1
	inc 	scH
no_carry_1:
		
	inc 	counter
		
	; Point to next sprite table entry		
	txa
	add 	#4
	tax

	jmp 	scr_loop

scr_done:

	; "Move" the scroller one pixel left	

	lda 	scrollX
	sub 	#1
	sta 	scrollX
	bcs 	no_carry2
	dec 	scrollXH
no_carry2:

	; Check if we've reached the final position of the current string	
	lda 	scrollXH
	cmp 	#$FF
	bne 	no_str_change
	

	ldx 	curr_str
	lda 	strstop,x
	cmp 	scrollX
	bne 	no_str_change

	inx
	cpx 	#13
	bne 	not_maxxed
	ldx 	#0
not_maxxed:
	stx 	curr_str
	txa
	asl 	a
	tax

	lda 	strtable,x
	sta 	STRPTRL
	inx
	lda 	strtable,x
	sta 	STRPTRH
	
	ldx 	#255
	stx 	scrollX
	inx
	stx 	scrollXH
	
no_str_change:		
	rts





;-----------------
;     VBLANK
;-----------------
nmi:
	; Save registers
	pha
	txa
	pha
	tya
	pha

	; DMA sprite data from page 4
	dma_sprites 4
	
	; Fade logo color
	dec 	fade_ctr
	bne 	no_fade_update
	lda 	#16
	sta 	fade_ctr
	lda 	fade_val
	tay
	tax
	iny
	cpy 	#6
	bne 	fade_val_ok
	ldy 	#0
fade_val_ok:
	sty 	fade_val
	lda 	fade_table,x
       	sta 	$581
no_fade_update:

	; Refresh palette data
	jsr 	refresh_palette

	; Draw vector balls
	ldx 	counter6
	lda 	sincub,x
	sta 	counter7

draw_balls:
	ldx 	counter5

	draw_ball 0
	draw_ball 1
	draw_ball 2
	draw_ball 3
	draw_ball 4
	draw_ball 5
	draw_ball 6
	draw_ball 7

	lda 	speed
	clc
	adc 	counter5
	sta 	counter5
	inc 	counter6

	; Update cube rotation speed 
	dec 	counter9
	bne 	nospeedchg
	lda 	#128
	sta 	counter9
	inc 	counter8
	lda 	counter8
	and 	#31
	sta 	counter8
	tax

	lda 	spin,x
	sta 	speed
nospeedchg:
	
	; Cycle text color
	dec 	counter3
	bne 	c3nz
	lda 	#40

	sta 	counter3
	ldx 	counter4
	inx
	cpx 	#12
	bne 	c4ok
	ldx 	#0
c4ok:
	stx 	counter4
	lda 	colcycle2,x
	sta 	$2D1
	lda 	colcycle1,x
	sta 	$2D2
c3nz:


	; Draw volume histograms
	lda 	#0
	sta 	counter10
	sta 	counter11
drawhist:
	ldx 	counter10
	ldy 	counter11
	lda 	vtx,x
	sta 	$4A3,y
	sta 	$4A7,y

	; current_envelope + n*2 contains the current volume of the n:th channel.
	; These values are set by the music player. Because of the characteristics
	; of channel 2 (triangle wave) it will have a volume of either 0 or 15.
	txa
	asl 	a
	tax
	lda 	current_envelope,x
	and 	#15
		
	asl 	a
	add 	#(99+29)
	sta 	$4A1,y
	add 	#1
	sta 	$4A5,y
		
	lda 	#2
	ora 	counter10
	sta 	$4A2,y
	sta 	$4A6,y

	lda 	#68
	sta 	$4A0,y
	lda 	#76
	sta 	$4A4,y
		
	tya
	add 	#8
	sta 	counter11
	inc 	counter10
	ldx 	counter10
	cpx 	#4
	bne 	drawhist
		
		
	; Reset VRAM address
	set_vram_adr $2000
	
	movi 	permit_read,1

	lda 	#(PPUCNT1_E_NMI | PPUCNT1_BG_0000 | PPUCNT1_OBJ_1000 | PPUCNT1_INC_1 | PPUCNT1_NT_2000)
	sta 	REG_PPUCNT1
	
	; Restore registers	
	pla
	tay
	pla
	tax
	pla
	

irq:
	rti

	

;------------------
;      DATA
;------------------

; 256 bytes
nametable_1:
	.incbin "data/chaddict2.nam"

; Attribute table data. 16 bytes (256*64 pixels = 8*2 attributes)
attrtable_1:
	.incbin "data/chaddict2.atr"


sprite_data:
	; Sprite 0
	.db 1,0,32,248
	

palette:
	.db 2,36,2,32		; 
	.db 2,14,32,38		; 
	.db 2,3,32,10		; 
	.db 2,3,32,9		; 

	.db 2,49,33,32		; 
	.db 2,19,35,48		; 
	.db 2,19,19,35		; 
	.db 2,51,52,51		; 

; Used for fading the logo
fade_table:
	.db 4,20,36,52,36,20


; Distortion table for the logo
sin_64_12:
	.db 1,1,2,2,3,3,4,4,5,5,6,6,5,5,5,6
	.db 6,6,6,5,5,4,4,5,5,6,6,5,5,4,4,3
	.db 3,2,2,1,1,2,2,3,3,3,2,2,1,1,1,2
	.db 2,2,3,3,3,3,2,2,2,1,1,1,1,2,2,2


; Used to displace the scrolling text verically	
sin256:
	.db 193,193,194,195,196,196,197,198,199,199,200,201,201,202,203,204
	.db 204,205,206,206,207,208,208,209,210,210,211,212,212,213,213,214
	.db 214,215,215,216,216,217,217,218,218,219,219,219,220,220,221,221
	.db 221,221,222,222,222,222,223,223,223,223,223,223,223,223,223,223
	.db 223,223,223,223,223,223,223,223,223,223,223,222,222,222,222,221
	.db 221,221,221,220,220,219,219,219,218,218,217,217,216,216,215,215
	.db 214,214,213,213,212,212,211,210,210,209,208,208,207,206,206,205
	.db 204,204,203,202,201,201,200,199,199,198,197,196,196,195,194,193
	.db 193,192,191,190,189,189,188,187,186,186,185,184,184,183,182,181
	.db 181,180,179,179,178,177,177,176,175,175,174,173,173,172,172,171
	.db 171,170,170,169,169,168,168,167,167,166,166,166,165,165,164,164
	.db 164,164,163,163,163,163,162,162,162,162,162,162,162,162,162,162
	.db 162,162,162,162,162,162,162,162,162,162,162,163,163,163,163,164
	.db 164,164,164,165,165,166,166,166,167,167,168,168,169,169,170,170
	.db 171,171,172,172,173,173,174,175,175,176,177,177,178,179,179,180
	.db 181,181,182,183,184,184,185,186,186,187,188,189,189,190,191,192
	
	
	
; Color pairs used by the scroller (3rd color is always 32=white)
colcycle1:
	.db 33,34,35,36,37,38,39,40,41,42,43,44
colcycle2:
	.db 49,50,51,52,53,54,55,56,57,58,59,60

	
szMessage: 
	.db "            mic presents:"
	.db 0
szMessage2:
	.db "2008  "
	.db 0
szMessage3:
	.db "'Chip Addiction'"
	.db 0
szMessage4:
	.db "Now with NTSC compability"
	.db 0
szMessage5:
	.db "40kB of NES juice"
	.db 0
szMessage6:
	.db "It's ok. It's demo for goat"
	.db 0
szMessage7:
	.db "..NES love that goat"
	.db 0
szMessage8:
	.db "Original groove by Mad Max"

	.db 0
szMessage9:
	.db "aesthetically abused in mml"

	.db 0
szMessage10:
	.db "Fancy the vector balls?"
	.db 0
szMessage11:
	.db "*cough* precalc *cough* :)"
	.db 0
szMessage12:
	.db "Enjoy the spectacle"

	.db 0
szMessage13:
	.db "<WRAP>"
	.db 0
	
strtable:
	.dw szMessage
	.dw szMessage2
	.dw szMessage3
	.dw szMessage4
	.dw szMessage5
	.dw szMessage6
	.dw szMessage7
	.dw szMessage8
	.dw szMessage9
	.dw szMessage10
	.dw szMessage11
	.dw szMessage12
	.dw szMessage13


; Skip to next string when the current position is $FFnn, where
; nn comes from this table
strstop:
	.db 20,166,100,36,85,5,36,16,4,36,10,60,194
	

; Look-up tables for the cube..
xlut:				;\
	.incbin "data/xlut.bin"	;  \
ylut:				;  / Unique x-y pairs
	.incbin "data/ylut.bin"	;/
idx0:
	.incbin "data/idx0.bin"	; x/ylut indices for vertex 0
idx1:
	.incbin "data/idx1.bin"	; ... for vertex 1
idx2:
	.incbin "data/idx2.bin"	; etc
idx3:
	.incbin "data/idx3.bin"
idx4:
	.incbin "data/idx4.bin"
idx5:
	.incbin "data/idx5.bin"
idx6:
	.incbin "data/idx6.bin"
idx7:
	.incbin "data/idx7.bin"


	

; Used for moving the cube horizontally
sincub:
	.db 128,129,130,132,133,135,136,138,139,141,142,144,145,146,148,149
	.db 150,152,153,154,156,157,158,160,161,162,163,164,166,167,168,169
	.db 170,171,172,173,174,175,176,177,177,178,179,180,180,181,182,182
	.db 183,183,184,184,185,185,186,186,186,187,187,187,187,187,187,187
	.db 187,187,187,187,187,187,187,187,186,186,186,185,185,184,184,183
	.db 183,182,182,181,180,180,179,178,177,177,176,175,174,173,172,171
	.db 170,169,168,167,166,164,163,162,161,160,158,157,156,154,153,152
	.db 150,149,148,146,145,144,142,141,139,138,136,135,133,132,130,129
	.db 128,126,125,123,122,120,119,117,116,114,113,111,110,109,107,106
	.db 105,103,102,101,99,98,97,95,94,93,92,91,89,88,87,86
	.db 85,84,83,82,81,80,79,78,78,77,76,75,75,74,73,73
	.db 72,72,71,71,70,70,69,69,69,68,68,68,68,68,68,68
	.db 68,68,68,68,68,68,68,68,69,69,69,70,70,71,71,72
	.db 72,73,73,74,75,75,76,77,78,78,79,80,81,82,83,84
	.db 85,86,87,88,89,91,92,93,94,95,97,98,99,101,102,103
	.db 105,106,107,109,110,111,113,114,116,117,119,120,122,123,125,126

; Rotation speeds for the cube
spin:
	.db 1,2,1,2,3,4,3,2,1,4,6,3,2,1,-1,1
	.db -1,-2,-4,-6,-7,6,5,4,3,2,1,-1,1,2,1,1



; Horizontal positions of the volume histograms
vtx:
	.db 8,17,231,240
chnr:
	.db 0,3,2,1
	

	
; Setup interrupt vectors
 .bank 3
 .org  $fffa
	.dw   nmi
	.dw   reset
	.dw   irq



;-----------------------
;       CHR-ROM
;-----------------------

 .bank 4
 .org $0000

	; Background image. 142 tiles
	; 157
	.incbin "gfx/chaddict2.pat"
 
 
 	; Tile consisting only of color 3 (which is black in all BG palettes)
 	black_tile:
 		.db $0,$0,$0,$0
 		.db $0,$0,$0,$0
 		.db $ff,$ff,$ff,$ff
 		.db $ff,$ff,$ff,$ff
 

 	; 12 tiles
 	.incbin "gfx/gradient8.pat"
 

 .org $1000
 	sprite0_pat:
  		.db $ff,$ff,$ff,$ff
  		.db $ff,$ff,$ff,$ff
  		.db $ff,$ff,$ff,$ff
 		.db $ff,$ff,$ff,$ff

	.incbin "gfx/ball.pat"
 		

 .org $1200
	; Font
	.incbin "gfx/adore64.pat"

	; Volume bars
	.incbin "gfx/bars.pat"

	.db 0,0,0,0,0,0,0,0
	.db 0,255,255,255,255,255,255,255
 	.db 0,0,0,0,0,0,0,0
	.db 255,255,255,255,255,255,255,255
  	
 	
 	
	
    